@@ -16,6 +16,7 @@ use uv_client::LineHaul;
 use uv_client::RegistryClientBuilder;
 use uv_pep508::{MarkerEnvironment, MarkerEnvironmentBuilder};
 use uv_platform_tags::{Arch, Os, Platform};
+use uv_redacted::DisplaySafeUrl;
 use uv_version::version;
 
 #[tokio::test]
@@ -54,12 +55,12 @@ async fn test_user_agent_has_version() -> Result<()> {
     let client = RegistryClientBuilder::new(cache).build();
 
     // Send request to our dummy server
-    let url = Url::from_str(&format!("http://{addr}"))?;
+    let url = DisplaySafeUrl::from_str(&format!("http://{addr}"))?;
     let res = client
         .cached_client()
         .uncached()
         .for_host(&url)
-        .get(url)
+        .get(Url::from(url))
         .send()
         .await?;
 
@@ -151,12 +152,12 @@ async fn test_user_agent_has_linehaul() -> Result<()> {
     let client = builder.build();
 
     // Send request to our dummy server
-    let url = Url::from_str(&format!("http://{addr}"))?;
+    let url = DisplaySafeUrl::from_str(&format!("http://{addr}"))?;
     let res = client
         .cached_client()
         .uncached()
         .for_host(&url)
-        .get(url)
+        .get(Url::from(url))
         .send()
         .await?;
 
