@@ -1,7 +1,6 @@
 pub use authentication::*;
 pub use build_options::*;
 pub use concurrency::*;
-pub use config_settings::*;
 pub use constraints::*;
 pub use dependency_groups::*;
 pub use dry_run::*;
@@ -15,10 +14,10 @@ pub use overrides::*;
 pub use package_options::*;
 pub use preview::*;
 pub use project_build_backend::*;
-pub use rayon::*;
 pub use required_version::*;
 pub use sources::*;
 pub use target_triple::*;
+pub use threading::*;
 pub use trusted_host::*;
 pub use trusted_publishing::*;
 pub use vcs::*;
@@ -26,7 +25,6 @@ pub use vcs::*;
 mod authentication;
 mod build_options;
 mod concurrency;
-mod config_settings;
 mod constraints;
 mod dependency_groups;
 mod dry_run;
@@ -40,10 +38,10 @@ mod overrides;
 mod package_options;
 mod preview;
 mod project_build_backend;
-mod rayon;
 mod required_version;
 mod sources;
 mod target_triple;
+mod threading;
 mod trusted_host;
 mod trusted_publishing;
 mod vcs;
