@@ -57,6 +57,7 @@ pub use crate::any::*;
 pub use crate::build_requires::*;
 pub use crate::buildable::*;
 pub use crate::cached::*;
+pub use crate::config_settings::*;
 pub use crate::dependency_metadata::*;
 pub use crate::diagnostic::*;
 pub use crate::dist_error::*;
@@ -86,6 +87,7 @@ mod any;
 mod build_requires;
 mod buildable;
 mod cached;
+mod config_settings;
 mod dependency_metadata;
 mod diagnostic;
 mod dist_error;
