@@ -1,10 +1,12 @@
 use std::borrow::Cow;
+
 use uv_cache::{Cache, CacheBucket, CacheShard, WheelCache};
 use uv_cache_info::CacheInfo;
 use uv_cache_key::cache_digest;
-use uv_configuration::{ConfigSettings, PackageConfigSettings};
 use uv_distribution_types::{
-    DirectUrlSourceDist, DirectorySourceDist, GitSourceDist, Hashed, PathSourceDist,
+    BuildVariables, ConfigSettings, DirectUrlSourceDist, DirectorySourceDist,
+    ExtraBuildRequirement, ExtraBuildRequires, ExtraBuildVariables, GitSourceDist, Hashed,
+    PackageConfigSettings, PathSourceDist,
 };
 use uv_normalize::PackageName;
 use uv_platform_tags::Tags;
@@ -22,6 +24,8 @@ pub struct BuiltWheelIndex<'a> {
     hasher: &'a HashStrategy,
     config_settings: &'a ConfigSettings,
     config_settings_package: &'a PackageConfigSettings,
+    extra_build_requires: &'a ExtraBuildRequires,
+    extra_build_variables: &'a ExtraBuildVariables,
 }
 
 impl<'a> BuiltWheelIndex<'a> {
@@ -32,6 +36,8 @@ impl<'a> BuiltWheelIndex<'a> {
         hasher: &'a HashStrategy,
         config_settings: &'a ConfigSettings,
         config_settings_package: &'a PackageConfigSettings,
+        extra_build_requires: &'a ExtraBuildRequires,
+        extra_build_variables: &'a ExtraBuildVariables,
     ) -> Self {
         Self {
             cache,
@@ -39,6 +45,8 @@ impl<'a> BuiltWheelIndex<'a> {
             hasher,
             config_settings,
             config_settings_package,
+            extra_build_requires,
+            extra_build_variables,
         }
     }
 
@@ -69,10 +77,19 @@ impl<'a> BuiltWheelIndex<'a> {
 
         // If there are build settings, we need to scope to a cache shard.
         let config_settings = self.config_settings_for(&source_dist.name);
-        let cache_shard = if config_settings.is_empty() {
+        let extra_build_deps = self.extra_build_requires_for(&source_dist.name);
+        let extra_build_vars = self.extra_build_variables_for(&source_dist.name);
+        let cache_shard = if config_settings.is_empty()
+            && extra_build_deps.is_empty()
+            && extra_build_vars.is_none()
+        {
             cache_shard
         } else {
-            cache_shard.shard(cache_digest(&config_settings))
+            cache_shard.shard(cache_digest(&(
+                &config_settings,
+                extra_build_deps,
+                extra_build_vars,
+            )))
         };
 
         Ok(self.find(&cache_shard))
@@ -107,10 +124,19 @@ impl<'a> BuiltWheelIndex<'a> {
 
         // If there are build settings, we need to scope to a cache shard.
         let config_settings = self.config_settings_for(&source_dist.name);
-        let cache_shard = if config_settings.is_empty() {
+        let extra_build_deps = self.extra_build_requires_for(&source_dist.name);
+        let extra_build_vars = self.extra_build_variables_for(&source_dist.name);
+        let cache_shard = if config_settings.is_empty()
+            && extra_build_deps.is_empty()
+            && extra_build_vars.is_none()
+        {
             cache_shard
         } else {
-            cache_shard.shard(cache_digest(&config_settings))
+            cache_shard.shard(cache_digest(&(
+                &config_settings,
+                extra_build_deps,
+                extra_build_vars,
+            )))
         };
 
         Ok(self
@@ -156,10 +182,19 @@ impl<'a> BuiltWheelIndex<'a> {
 
         // If there are build settings, we need to scope to a cache shard.
         let config_settings = self.config_settings_for(&source_dist.name);
-        let cache_shard = if config_settings.is_empty() {
+        let extra_build_deps = self.extra_build_requires_for(&source_dist.name);
+        let extra_build_vars = self.extra_build_variables_for(&source_dist.name);
+        let cache_shard = if config_settings.is_empty()
+            && extra_build_deps.is_empty()
+            && extra_build_vars.is_none()
+        {
             cache_shard
         } else {
-            cache_shard.shard(cache_digest(&config_settings))
+            cache_shard.shard(cache_digest(&(
+                &config_settings,
+                extra_build_deps,
+                extra_build_vars,
+            )))
         };
 
         Ok(self
@@ -183,10 +218,19 @@ impl<'a> BuiltWheelIndex<'a> {
 
         // If there are build settings, we need to scope to a cache shard.
         let config_settings = self.config_settings_for(&source_dist.name);
-        let cache_shard = if config_settings.is_empty() {
+        let extra_build_deps = self.extra_build_requires_for(&source_dist.name);
+        let extra_build_vars = self.extra_build_variables_for(&source_dist.name);
+        let cache_shard = if config_settings.is_empty()
+            && extra_build_deps.is_empty()
+            && extra_build_vars.is_none()
+        {
             cache_shard
         } else {
-            cache_shard.shard(cache_digest(&config_settings))
+            cache_shard.shard(cache_digest(&(
+                &config_settings,
+                extra_build_deps,
+                extra_build_vars,
+            )))
         };
 
         self.find(&cache_shard)
@@ -257,4 +301,17 @@ impl<'a> BuiltWheelIndex<'a> {
             Cow::Borrowed(self.config_settings)
         }
     }
+
+    /// Determine the extra build requirements for the given package name.
+    fn extra_build_requires_for(&self, name: &PackageName) -> &[ExtraBuildRequirement] {
+        self.extra_build_requires
+            .get(name)
+            .map(Vec::as_slice)
+            .unwrap_or(&[])
+    }
+
+    /// Determine the extra build variables for the given package name.
+    fn extra_build_variables_for(&self, name: &PackageName) -> Option<&BuildVariables> {
+        self.extra_build_variables.get(name)
+    }
 }
