@@ -1,25 +1,29 @@
-use anyhow::{bail, Result};
 use std::sync::Arc;
+
+use anyhow::{Result, bail};
+use owo_colors::OwoColorize;
 use tracing::{debug, warn};
 
 use uv_cache::{Cache, CacheBucket, WheelCache};
 use uv_cache_info::Timestamp;
-use uv_configuration::{BuildOptions, ConfigSettings, Reinstall};
+use uv_configuration::{BuildOptions, Reinstall};
 use uv_distribution::{
     BuiltWheelIndex, HttpArchivePointer, LocalArchivePointer, RegistryWheelIndex,
 };
+use uv_distribution_filename::WheelFilename;
 use uv_distribution_types::{
-    BuiltDist, CachedDirectUrlDist, CachedDist, Dist, Error, Hashed, IndexLocations, InstalledDist,
-    Name, RequirementSource, Resolution, ResolvedDist, SourceDist,
+    BuiltDist, CachedDirectUrlDist, CachedDist, ConfigSettings, Dist, Error, ExtraBuildRequires,
+    ExtraBuildVariables, Hashed, IndexLocations, InstalledDist, Name, PackageConfigSettings,
+    RequirementSource, Resolution, ResolvedDist, SourceDist,
 };
 use uv_fs::Simplified;
-use uv_platform_tags::Tags;
+use uv_platform_tags::{IncompatibleTag, TagCompatibility, Tags};
 use uv_pypi_types::VerbatimParsedUrl;
 use uv_python::PythonEnvironment;
 use uv_types::HashStrategy;
 
-use crate::satisfies::RequirementSatisfaction;
 use crate::SitePackages;
+use crate::satisfies::RequirementSatisfaction;
 
 /// A planner to generate an [`Plan`] based on a set of requirements.
 #[derive(Debug)]
@@ -52,14 +56,33 @@ impl<'a> Planner<'a> {
         hasher: &HashStrategy,
         index_locations: &IndexLocations,
         config_settings: &ConfigSettings,
+        config_settings_package: &PackageConfigSettings,
+        extra_build_requires: &ExtraBuildRequires,
+        extra_build_variables: &ExtraBuildVariables,
         cache: &Cache,
         venv: &PythonEnvironment,
         tags: &Tags,
     ) -> Result<Plan> {
         // Index all the already-downloaded wheels in the cache.
-        let mut registry_index =
-            RegistryWheelIndex::new(cache, tags, index_locations, hasher, config_settings);
-        let built_index = BuiltWheelIndex::new(cache, tags, hasher, config_settings);
+        let mut registry_index = RegistryWheelIndex::new(
+            cache,
+            tags,
+            index_locations,
+            hasher,
+            config_settings,
+            config_settings_package,
+            extra_build_requires,
+            extra_build_variables,
+        );
+        let built_index = BuiltWheelIndex::new(
+            cache,
+            tags,
+            hasher,
+            config_settings,
+            config_settings_package,
+            extra_build_requires,
+            extra_build_variables,
+        );
 
         let mut cached = vec![];
         let mut remote = vec![];
@@ -162,6 +185,16 @@ impl<'a> Planner<'a> {
                 }
                 Dist::Built(BuiltDist::DirectUrl(wheel)) => {
                     if !wheel.filename.is_compatible(tags) {
+                        let hint = generate_wheel_compatibility_hint(&wheel.filename, tags);
+                        if let Some(hint) = hint {
+                            bail!(
+                                "A URL dependency is incompatible with the current platform: {}\n\n{}{} {}",
+                                wheel.url,
+                                "hint".bold().cyan(),
+                                ":".bold(),
+                                hint
+                            );
+                        }
                         bail!(
                             "A URL dependency is incompatible with the current platform: {}",
                             wheel.url
@@ -224,6 +257,16 @@ impl<'a> Planner<'a> {
                     }
 
                     if !wheel.filename.is_compatible(tags) {
+                        let hint = generate_wheel_compatibility_hint(&wheel.filename, tags);
+                        if let Some(hint) = hint {
+                            bail!(
+                                "A path dependency is incompatible with the current platform: {}\n\n{}{} {}",
+                                wheel.install_path.user_display(),
+                                "hint".bold().cyan(),
+                                ":".bold(),
+                                hint
+                            );
+                        }
                         bail!(
                             "A path dependency is incompatible with the current platform: {}",
                             wheel.install_path.user_display()
@@ -459,6 +502,132 @@ fn is_seed_package(dist_info: &InstalledDist, venv: &PythonEnvironment) -> bool
     }
 }
 
+/// Generate a hint for explaining wheel compatibility issues.
+fn generate_wheel_compatibility_hint(filename: &WheelFilename, tags: &Tags) -> Option<String> {
+    let TagCompatibility::Incompatible(incompatible_tag) = filename.compatibility(tags) else {
+        return None;
+    };
+
+    match incompatible_tag {
+        IncompatibleTag::Python => {
+            let wheel_tags = filename.python_tags();
+            let current_tag = tags.python_tag();
+
+            if let Some(current) = current_tag {
+                let message = if let Some(pretty) = current.pretty() {
+                    format!("{} (`{}`)", pretty.cyan(), current.cyan())
+                } else {
+                    format!("`{}`", current.cyan())
+                };
+
+                Some(format!(
+                    "The wheel is compatible with {}, but you're using {}",
+                    wheel_tags
+                        .iter()
+                        .map(|tag| if let Some(pretty) = tag.pretty() {
+                            format!("{} (`{}`)", pretty.cyan(), tag.cyan())
+                        } else {
+                            format!("`{}`", tag.cyan())
+                        })
+                        .collect::<Vec<_>>()
+                        .join(", "),
+                    message
+                ))
+            } else {
+                Some(format!(
+                    "The wheel requires {}",
+                    wheel_tags
+                        .iter()
+                        .map(|tag| if let Some(pretty) = tag.pretty() {
+                            format!("{} (`{}`)", pretty.cyan(), tag.cyan())
+                        } else {
+                            format!("`{}`", tag.cyan())
+                        })
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                ))
+            }
+        }
+        IncompatibleTag::Abi => {
+            let wheel_tags = filename.abi_tags();
+            let current_tag = tags.abi_tag();
+
+            if let Some(current) = current_tag {
+                let message = if let Some(pretty) = current.pretty() {
+                    format!("{} (`{}`)", pretty.cyan(), current.cyan())
+                } else {
+                    format!("`{}`", current.cyan())
+                };
+                Some(format!(
+                    "The wheel is compatible with {}, but you're using {}",
+                    wheel_tags
+                        .iter()
+                        .map(|tag| if let Some(pretty) = tag.pretty() {
+                            format!("{} (`{}`)", pretty.cyan(), tag.cyan())
+                        } else {
+                            format!("`{}`", tag.cyan())
+                        })
+                        .collect::<Vec<_>>()
+                        .join(", "),
+                    message
+                ))
+            } else {
+                Some(format!(
+                    "The wheel requires {}",
+                    wheel_tags
+                        .iter()
+                        .map(|tag| if let Some(pretty) = tag.pretty() {
+                            format!("{} (`{}`)", pretty.cyan(), tag.cyan())
+                        } else {
+                            format!("`{}`", tag.cyan())
+                        })
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                ))
+            }
+        }
+        IncompatibleTag::Platform => {
+            let wheel_tags = filename.platform_tags();
+            let current_tag = tags.platform_tag();
+
+            if let Some(current) = current_tag {
+                let message = if let Some(pretty) = current.pretty() {
+                    format!("{} (`{}`)", pretty.cyan(), current.cyan())
+                } else {
+                    format!("`{}`", current.cyan())
+                };
+                Some(format!(
+                    "The wheel is compatible with {}, but you're on {}",
+                    wheel_tags
+                        .iter()
+                        .map(|tag| if let Some(pretty) = tag.pretty() {
+                            format!("{} (`{}`)", pretty.cyan(), tag.cyan())
+                        } else {
+                            format!("`{}`", tag.cyan())
+                        })
+                        .collect::<Vec<_>>()
+                        .join(", "),
+                    message
+                ))
+            } else {
+                Some(format!(
+                    "The wheel requires {}",
+                    wheel_tags
+                        .iter()
+                        .map(|tag| if let Some(pretty) = tag.pretty() {
+                            format!("{} (`{}`)", pretty.cyan(), tag.cyan())
+                        } else {
+                            format!("`{}`", tag.cyan())
+                        })
+                        .collect::<Vec<_>>()
+                        .join(", ")
+                ))
+            }
+        }
+        _ => None,
+    }
+}
+
 #[derive(Debug, Default)]
 pub struct Plan {
     /// The distributions that are not already installed in the current environment, but are
