@@ -12,6 +12,7 @@ use std::fmt::Display;
 use thiserror::Error;
 use tracing::{debug, trace};
 use url::Url;
+use uv_redacted::DisplaySafeUrl;
 use uv_static::EnvVars;
 
 #[derive(Debug, Error)]
@@ -23,9 +24,9 @@ pub enum TrustedPublishingError {
     #[error(transparent)]
     Url(#[from] url::ParseError),
     #[error("Failed to fetch: `{0}`")]
-    Reqwest(Url, #[source] reqwest::Error),
+    Reqwest(DisplaySafeUrl, #[source] reqwest::Error),
     #[error("Failed to fetch: `{0}`")]
-    ReqwestMiddleware(Url, #[source] reqwest_middleware::Error),
+    ReqwestMiddleware(DisplaySafeUrl, #[source] reqwest_middleware::Error),
     #[error(transparent)]
     SerdeJson(#[from] serde_json::error::Error),
     #[error(
@@ -94,7 +95,7 @@ pub struct OidcTokenClaims {
 
 /// Returns the short-lived token to use for uploading.
 pub(crate) async fn get_token(
-    registry: &Url,
+    registry: &DisplaySafeUrl,
     client: &ClientWithMiddleware,
 ) -> Result<TrustedPublishingToken, TrustedPublishingError> {
     // If this fails, we can skip the audience request.
@@ -124,15 +125,16 @@ pub(crate) async fn get_token(
 }
 
 async fn get_audience(
-    registry: &Url,
+    registry: &DisplaySafeUrl,
     client: &ClientWithMiddleware,
 ) -> Result<String, TrustedPublishingError> {
     // `pypa/gh-action-pypi-publish` uses `netloc` (RFC 1808), which is deprecated for authority
     // (RFC 3986).
-    let audience_url = Url::parse(&format!("https://{}/_/oidc/audience", registry.authority()))?;
+    let audience_url =
+        DisplaySafeUrl::parse(&format!("https://{}/_/oidc/audience", registry.authority()))?;
     debug!("Querying the trusted publishing audience from {audience_url}");
     let response = client
-        .get(audience_url.clone())
+        .get(Url::from(audience_url.clone()))
         .send()
         .await
         .map_err(|err| TrustedPublishingError::ReqwestMiddleware(audience_url.clone(), err))?;
@@ -154,14 +156,14 @@ async fn get_oidc_token(
     let oidc_token_url = env::var(EnvVars::ACTIONS_ID_TOKEN_REQUEST_URL).map_err(|err| {
         TrustedPublishingError::from_var_err(EnvVars::ACTIONS_ID_TOKEN_REQUEST_URL, err)
     })?;
-    let mut oidc_token_url = Url::parse(&oidc_token_url)?;
+    let mut oidc_token_url = DisplaySafeUrl::parse(&oidc_token_url)?;
     oidc_token_url
         .query_pairs_mut()
         .append_pair("audience", audience);
     debug!("Querying the trusted publishing OIDC token from {oidc_token_url}");
     let authorization = format!("bearer {oidc_token_request_token}");
     let response = client
-        .get(oidc_token_url.clone())
+        .get(Url::from(oidc_token_url.clone()))
         .header(header::AUTHORIZATION, authorization)
         .send()
         .await
@@ -188,11 +190,11 @@ fn decode_oidc_token(oidc_token: &str) -> Option<OidcTokenClaims> {
 }
 
 async fn get_publish_token(
-    registry: &Url,
+    registry: &DisplaySafeUrl,
     oidc_token: &str,
     client: &ClientWithMiddleware,
 ) -> Result<TrustedPublishingToken, TrustedPublishingError> {
-    let mint_token_url = Url::parse(&format!(
+    let mint_token_url = DisplaySafeUrl::parse(&format!(
         "https://{}/_/oidc/mint-token",
         registry.authority()
     ))?;
@@ -201,7 +203,7 @@ async fn get_publish_token(
         token: oidc_token.to_string(),
     };
     let response = client
-        .post(mint_token_url.clone())
+        .post(Url::from(mint_token_url.clone()))
         .body(serde_json::to_vec(&mint_token_payload)?)
         .send()
         .await
