@@ -53,8 +53,7 @@ import re
 from dataclasses import asdict, dataclass, field
 from enum import StrEnum
 from pathlib import Path
-from typing import Generator, Iterable, NamedTuple, Self
-from urllib.parse import unquote
+from typing import Any, Generator, Iterable, NamedTuple, Self
 
 import httpx
 
@@ -255,8 +254,7 @@ class CPythonFinder(Finder):
                 # Sort the assets to ensure deterministic results
                 row["assets"].sort(key=lambda asset: asset["browser_download_url"])
                 for asset in row["assets"]:
-                    url = asset["browser_download_url"]
-                    download = self._parse_download_url(url)
+                    download = self._parse_download_asset(asset)
                     if download is None:
                         continue
                     if (
@@ -305,6 +303,9 @@ class CPythonFinder(Finder):
         """Fetch the checksums for the given downloads."""
         checksum_urls = set()
         for download in downloads:
+            # Skip the newer releases where we got the hash from the GitHub API
+            if download.sha256:
+                continue
             release_base_url = download.url.rsplit("/", maxsplit=1)[0]
             checksum_url = release_base_url + "/SHA256SUMS"
             checksum_urls.add(checksum_url)
@@ -343,16 +344,23 @@ class CPythonFinder(Finder):
                 checksums[filename] = checksum
 
         for download in downloads:
+            if download.sha256:
+                continue
             download.sha256 = checksums.get(download.filename)
 
-    def _parse_download_url(self, url: str) -> PythonDownload | None:
-        """Parse an indygreg download URL into a PythonDownload object."""
+    def _parse_download_asset(self, asset: dict[str, Any]) -> PythonDownload | None:
+        """Parse a python-build-standalone download asset into a PythonDownload object."""
+        url = asset["browser_download_url"]
         # Ex)
         # https://github.com/astral-sh/python-build-standalone/releases/download/20240107/cpython-3.12.1%2B20240107-aarch64-unknown-linux-gnu-lto-full.tar.zst
         if url.endswith(".sha256"):
             return None
-        filename = unquote(url.rsplit("/", maxsplit=1)[-1])
         release = int(url.rsplit("/")[-2])
+        filename = asset["name"]
+        sha256 = None
+        # On older versions, GitHub didn't backfill the digest.
+        if digest := asset["digest"]:
+            sha256 = digest.removeprefix("sha256:")
 
         match = self._filename_re.match(filename) or self._legacy_filename_re.match(
             filename
@@ -391,6 +399,7 @@ class CPythonFinder(Finder):
             url=url,
             build_options=build_options,
             variant=variant,
+            sha256=sha256,
         )
 
     def _normalize_triple(self, triple: str) -> PlatformTriple | None:
@@ -541,6 +550,92 @@ class PyPyFinder(Finder):
             download.sha256 = checksums.get(download.filename)
 
 
+class PyodideFinder(Finder):
+    implementation = ImplementationName.CPYTHON
+
+    RELEASE_URL = "https://api.github.com/repos/pyodide/pyodide/releases"
+    METADATA_URL = (
+        "https://pyodide.github.io/pyodide/api/pyodide-cross-build-environments.json"
+    )
+
+    TRIPLE = PlatformTriple(
+        platform="emscripten",
+        arch=Arch("wasm32"),
+        libc="musl",
+    )
+
+    def __init__(self, client: httpx.AsyncClient):
+        self.client = client
+
+    async def find(self) -> list[PythonDownload]:
+        downloads = await self._fetch_downloads()
+        await self._fetch_checksums(downloads, n=10)
+        return downloads
+
+    async def _fetch_downloads(self) -> list[PythonDownload]:
+        # This will only download the first page, i.e., ~30 releases
+        [release_resp, meta_resp] = await asyncio.gather(
+            self.client.get(self.RELEASE_URL), self.client.get(self.METADATA_URL)
+        )
+        release_resp.raise_for_status()
+        meta_resp.raise_for_status()
+        releases = release_resp.json()
+        metadata = meta_resp.json()["releases"]
+
+        maj_minor_seen = set()
+        results = []
+        for release in releases:
+            pyodide_version = release["tag_name"]
+            meta = metadata.get(pyodide_version, None)
+            if meta is None:
+                continue
+
+            maj_min = pyodide_version.rpartition(".")[0]
+            # Only keep latest
+            if maj_min in maj_minor_seen:
+                continue
+            maj_minor_seen.add(maj_min)
+
+            python_version = Version.from_str(meta["python_version"])
+            # Find xbuildenv asset
+            for asset in release["assets"]:
+                if asset["name"].startswith("xbuildenv"):
+                    break
+
+            url = asset["browser_download_url"]
+            results.append(
+                PythonDownload(
+                    release=0,
+                    version=python_version,
+                    triple=self.TRIPLE,
+                    flavor=pyodide_version,
+                    implementation=self.implementation,
+                    filename=asset["name"],
+                    url=url,
+                )
+            )
+
+        return results
+
+    async def _fetch_checksums(self, downloads: list[PythonDownload], n: int) -> None:
+        for idx, batch in enumerate(batched(downloads, n)):
+            logging.info("Fetching Pyodide checksums: %d/%d", idx * n, len(downloads))
+            checksum_requests = []
+            for download in batch:
+                url = download.url + ".sha256"
+                checksum_requests.append(self.client.get(url))
+            for download, resp in zip(
+                batch, await asyncio.gather(*checksum_requests), strict=False
+            ):
+                try:
+                    resp.raise_for_status()
+                except httpx.HTTPStatusError as e:
+                    if e.response.status_code == 404:
+                        continue
+                    raise
+                download.sha256 = resp.text.strip()
+
+
 class GraalPyFinder(Finder):
     implementation = ImplementationName.GRAALPY
 
@@ -598,6 +693,9 @@ class GraalPyFinder(Finder):
                 platform = self._normalize_os(m.group(1))
                 arch = self._normalize_arch(m.group(2))
                 libc = "gnu" if platform == "linux" else "none"
+                sha256 = None
+                if digest := asset["digest"]:
+                    sha256 = digest.removeprefix("sha256:")
                 download = PythonDownload(
                     release=0,
                     version=python_version,
@@ -610,6 +708,7 @@ class GraalPyFinder(Finder):
                     implementation=self.implementation,
                     filename=asset["name"],
                     url=url,
+                    sha256=sha256,
                 )
                 # Only keep the latest GraalPy version of each arch/platform
                 if (python_version, arch, platform) not in results:
@@ -624,13 +723,16 @@ class GraalPyFinder(Finder):
         return self.PLATFORM_MAPPING.get(os, os)
 
     async def _fetch_checksums(self, downloads: list[PythonDownload], n: int) -> None:
+        downloads = list(filter(lambda d: not d.sha256, downloads))
         for idx, batch in enumerate(batched(downloads, n)):
             logging.info("Fetching GraalPy checksums: %d/%d", idx * n, len(downloads))
             checksum_requests = []
             for download in batch:
                 url = download.url + ".sha256"
                 checksum_requests.append(self.client.get(url))
-            for download, resp in zip(batch, await asyncio.gather(*checksum_requests)):
+            for download, resp in zip(
+                batch, await asyncio.gather(*checksum_requests), strict=False
+            ):
                 try:
                     resp.raise_for_status()
                 except httpx.HTTPStatusError as e:
@@ -735,6 +837,7 @@ async def find() -> None:
         CPythonFinder(client),
         PyPyFinder(client),
         GraalPyFinder(client),
+        PyodideFinder(client),
     ]
     downloads = []
 
