@@ -4,6 +4,7 @@ Queries information about the current Python interpreter and prints it as JSON.
 The script will exit with status 0 on known error that are turned into rust errors.
 """
 
+import site
 import sys
 
 import json
@@ -39,10 +40,11 @@ if hasattr(sys, "implementation"):
         # GraalPy reports the CPython version as sys.implementation.version,
         # so we need to discover the GraalPy version from the cache_tag
         import re
+
         implementation_version = re.sub(
-            r"graalpy(\d)(\d+)-\d+",
+            r"graalpy(\d)(\d+)(?:dev[\da-f]+)?-\d+",
             r"\1.\2",
-            sys.implementation.cache_tag
+            sys.implementation.cache_tag,
         )
     else:
         implementation_version = format_full_version(sys.implementation.version)
@@ -583,7 +585,6 @@ def main() -> None:
     elif os_and_arch["os"]["name"] == "musllinux":
         manylinux_compatible = True
 
-
     # By default, pip uses sysconfig on Python 3.10+.
     # But Python distributors can override this decision by setting:
     #     sysconfig._PIP_USE_SYSCONFIG = True / False
@@ -608,7 +609,7 @@ def main() -> None:
             except (ImportError, AttributeError):
                 pass
 
-            import distutils.dist
+            import distutils.dist  # noqa: F401
         except ImportError:
             # We require distutils, but it's not installed; this is fairly
             # common in, e.g., deadsnakes where distutils is packaged
@@ -637,11 +638,15 @@ def main() -> None:
         # temporary path to `sys.path` so we can import it, which we have to strip later
         # to avoid having this now-deleted path around.
         "sys_path": sys.path[1:],
+        "site_packages": site.getsitepackages(),
         "stdlib": sysconfig.get_path("stdlib"),
         # Prior to the introduction of `sysconfig` patching, python-build-standalone installations would always use
         # "/install" as the prefix. With `sysconfig` patching, we rewrite the prefix to match the actual installation
         # location. So in newer versions, we also write a dedicated flag to indicate standalone builds.
-        "standalone": sysconfig.get_config_var("prefix") == "/install" or bool(sysconfig.get_config_var("PYTHON_BUILD_STANDALONE")),
+        "standalone": (
+            sysconfig.get_config_var("prefix") == "/install"
+            or bool(sysconfig.get_config_var("PYTHON_BUILD_STANDALONE"))
+        ),
         "scheme": get_scheme(use_sysconfig_scheme),
         "virtualenv": get_virtualenv(),
         "platform": os_and_arch,
