@@ -1,6 +1,6 @@
 use std::sync::Arc;
 
-use futures::{stream::FuturesOrdered, TryStreamExt};
+use futures::{TryStreamExt, stream::FuturesOrdered};
 
 use uv_distribution::{DistributionDatabase, Reporter};
 use uv_distribution_types::DistributionMetadata;
@@ -8,7 +8,7 @@ use uv_distribution_types::Requirement;
 use uv_resolver::{InMemoryIndex, MetadataResponse};
 use uv_types::{BuildContext, HashStrategy};
 
-use crate::{required_dist, Error};
+use crate::{Error, required_dist};
 
 /// A resolver to expand the requested extras for a set of requirements to include all defined
 /// extras.
@@ -55,7 +55,7 @@ impl<'a, Context: BuildContext> ExtrasResolver<'a, Context> {
             database,
         } = self;
         requirements
-            .map(|requirement| async {
+            .map(async |requirement| {
                 Self::resolve_requirement(requirement, hasher, index, &database).await
             })
             .collect::<FuturesOrdered<_>>()
