@@ -173,7 +173,7 @@ where
     PylockTomlErrorKind: From<E>,
 {
     fn from(err: E) -> Self {
-        PylockTomlError {
+        Self {
             kind: Box::new(PylockTomlErrorKind::from(err)),
             hint: None,
         }
@@ -601,7 +601,7 @@ impl<'lock> PylockToml {
         packages.sort_by(|a, b| a.name.cmp(&b.name).then(a.version.cmp(&b.version)));
 
         // Return the constructed `pylock.toml`.
-        Ok(PylockToml {
+        Ok(Self {
             lock_version,
             created_by,
             requires_python: Some(requires_python),
@@ -1306,7 +1306,7 @@ impl PylockTomlPackage {
 
 impl PylockTomlWheel {
     /// Return the [`WheelFilename`] for this wheel.
-    fn filename(&self, name: &PackageName) -> Result<Cow<WheelFilename>, PylockTomlErrorKind> {
+    fn filename(&self, name: &PackageName) -> Result<Cow<'_, WheelFilename>, PylockTomlErrorKind> {
         if let Some(name) = self.name.as_ref() {
             Ok(Cow::Borrowed(name))
         } else if let Some(path) = self.path.as_ref() {
@@ -1606,7 +1606,7 @@ impl PylockTomlArchive {
                 }
             }
         } else {
-            return Err(PylockTomlErrorKind::ArchiveMissingPathUrl(name.clone()));
+            Err(PylockTomlErrorKind::ArchiveMissingPathUrl(name.clone()))
         }
     }
 
@@ -1631,7 +1631,7 @@ impl PylockTomlArchive {
             let ext = DistExtension::from_path(filename)?;
             Ok(matches!(ext, DistExtension::Wheel))
         } else {
-            return Err(PylockTomlErrorKind::ArchiveMissingPathUrl(name.clone()));
+            Err(PylockTomlErrorKind::ArchiveMissingPathUrl(name.clone()))
         }
     }
 }
