@@ -13,7 +13,6 @@ use uv_configuration::ExtrasSpecificationWithDefaults;
 use uv_configuration::{BuildOptions, DependencyGroupsWithDefaults, InstallOptions};
 use uv_distribution_types::{Edge, Node, Resolution, ResolvedDist};
 use uv_normalize::{ExtraName, GroupName, PackageName};
-use uv_pep508::MarkerTree;
 use uv_platform_tags::Tags;
 use uv_pypi_types::ResolverMarkerEnvironment;
 
@@ -49,6 +48,7 @@ pub trait Installable<'lock> {
 
         let mut queue: VecDeque<(&Package, Option<&ExtraName>)> = VecDeque::new();
         let mut seen = FxHashSet::default();
+        let mut activated_projects: Vec<&PackageName> = vec![];
         let mut activated_extras: Vec<(&PackageName, &ExtraName)> = vec![];
         let mut activated_groups: Vec<(&PackageName, &GroupName)> = vec![];
 
@@ -75,6 +75,7 @@ pub trait Installable<'lock> {
 
                 // Track the activated extras.
                 if dev.prod() {
+                    activated_projects.push(&dist.id.name);
                     for extra in extras.extra_names(dist.optional_dependencies.keys()) {
                         activated_extras.push((&dist.id.name, extra));
                     }
@@ -113,7 +114,7 @@ pub trait Installable<'lock> {
             inverse.insert(&dist.id, index);
 
             // Add an edge from the root.
-            petgraph.add_edge(root, index, Edge::Prod(MarkerTree::TRUE));
+            petgraph.add_edge(root, index, Edge::Prod);
 
             // Push the package onto the queue.
             roots.push((dist, index));
@@ -144,6 +145,7 @@ pub trait Installable<'lock> {
             {
                 if !dep.complexified_marker.evaluate(
                     marker_env,
+                    activated_projects.iter().copied(),
                     activated_extras.iter().copied(),
                     activated_groups.iter().copied(),
                 ) {
@@ -189,7 +191,7 @@ pub trait Installable<'lock> {
                     // a specific marker environment and set of extras/groups.
                     // So at this point, we know the extras/groups have been
                     // satisfied, so we can safely drop the conflict marker.
-                    Edge::Dev(group.clone(), dep.complexified_marker.pep508()),
+                    Edge::Dev(group.clone()),
                 );
 
                 // Push its dependencies on the queue.
@@ -231,7 +233,7 @@ pub trait Installable<'lock> {
             inverse.insert(&dist.id, index);
 
             // Add the edge.
-            petgraph.add_edge(root, index, Edge::Prod(dependency.marker));
+            petgraph.add_edge(root, index, Edge::Prod);
 
             // Push its dependencies on the queue.
             if seen.insert((&dist.id, None)) {
@@ -300,7 +302,7 @@ pub trait Installable<'lock> {
             };
 
             // Add the edge.
-            petgraph.add_edge(root, index, Edge::Dev(group.clone(), dependency.marker));
+            petgraph.add_edge(root, index, Edge::Dev(group.clone()));
 
             // Push its dependencies on the queue.
             if seen.insert((&dist.id, None)) {
@@ -368,6 +370,7 @@ pub trait Installable<'lock> {
                     }
                     if !dep.complexified_marker.evaluate(
                         marker_env,
+                        activated_projects.iter().copied(),
                         activated_extras
                             .iter()
                             .chain(additional_activated_extras.iter())
@@ -455,6 +458,7 @@ pub trait Installable<'lock> {
             for dep in deps {
                 if !dep.complexified_marker.evaluate(
                     marker_env,
+                    activated_projects.iter().copied(),
                     activated_extras.iter().copied(),
                     activated_groups.iter().copied(),
                 ) {
@@ -484,9 +488,9 @@ pub trait Installable<'lock> {
                     index,
                     dep_index,
                     if let Some(extra) = extra {
-                        Edge::Optional(extra.clone(), dep.complexified_marker.pep508())
+                        Edge::Optional(extra.clone())
                     } else {
-                        Edge::Prod(dep.complexified_marker.pep508())
+                        Edge::Prod
                     },
                 );
 
