@@ -8,8 +8,8 @@ use uv_distribution_types::{Requirement, RequirementSource};
 use uv_normalize::{ExtraName, GroupName, PackageName};
 use uv_pep440::{Version, VersionSpecifiers};
 use uv_pypi_types::{
-    Conflicts, ParsedArchiveUrl, ParsedDirectoryUrl, ParsedGitUrl, ParsedPathUrl, ParsedUrl,
-    VerbatimParsedUrl,
+    ConflictItemRef, Conflicts, ParsedArchiveUrl, ParsedDirectoryUrl, ParsedGitUrl, ParsedPathUrl,
+    ParsedUrl, VerbatimParsedUrl,
 };
 
 use crate::pubgrub::{PubGrubPackage, PubGrubPackageInner};
@@ -19,6 +19,21 @@ pub(crate) struct PubGrubDependency {
     pub(crate) package: PubGrubPackage,
     pub(crate) version: Ranges<Version>,
 
+    /// When the parent that created this dependency is a "normal" package
+    /// (non-extra non-group), this corresponds to its name.
+    ///
+    /// This is used to create project-level `ConflictItemRef` for a specific
+    /// package. In effect, this lets us "delay" filtering of project
+    /// dependencies when a conflict is declared between the project and a
+    /// group.
+    ///
+    /// The main problem with dealing with project level conflicts is that if you
+    /// declare a conflict between a package and a group, we represent that
+    /// group as a dependency of that package. So if you filter out the package
+    /// in a fork due to a conflict, you also filter out the group. Therefore,
+    /// we introduce this parent field to enable "delayed" filtering.
+    pub(crate) parent: Option<PackageName>,
+
     /// This field is set if the [`Requirement`] had a URL. We still use a URL from [`Urls`]
     /// even if this field is None where there is an override with a URL or there is a different
     /// requirement or constraint for the same package that has a URL.
@@ -30,8 +45,12 @@ impl PubGrubDependency {
         conflicts: &Conflicts,
         requirement: Cow<'a, Requirement>,
         dev: Option<&'a GroupName>,
-        source_name: Option<&'a PackageName>,
+        parent_package: Option<&'a PubGrubPackage>,
     ) -> impl Iterator<Item = Self> + 'a {
+        let parent_name = parent_package.and_then(|package| package.name_no_root());
+        let is_normal_parent = parent_package
+            .map(|pp| pp.extra().is_none() && pp.dev().is_none())
+            .unwrap_or(false);
         let iter = if !requirement.extras.is_empty() {
             // This is crazy subtle, but if any of the extras in the
             // requirement are part of a declared conflict, then we
@@ -80,50 +99,59 @@ impl PubGrubDependency {
 
         // Add the package, plus any extra variants.
         iter.map(move |(extra, group)| {
-            PubGrubRequirement::from_requirement(&requirement, extra, group)
-        })
-        .map(move |requirement| {
+            let pubgrub_requirement =
+                PubGrubRequirement::from_requirement(&requirement, extra, group);
             let PubGrubRequirement {
                 package,
                 version,
                 url,
-            } = requirement;
+            } = pubgrub_requirement;
             match &*package {
-                PubGrubPackageInner::Package { .. } => PubGrubDependency {
+                PubGrubPackageInner::Package { .. } => Self {
                     package,
                     version,
+                    parent: if is_normal_parent {
+                        parent_name.cloned()
+                    } else {
+                        None
+                    },
                     url,
                 },
-                PubGrubPackageInner::Marker { .. } => PubGrubDependency {
+                PubGrubPackageInner::Marker { .. } => Self {
                     package,
                     version,
+                    parent: if is_normal_parent {
+                        parent_name.cloned()
+                    } else {
+                        None
+                    },
                     url,
                 },
                 PubGrubPackageInner::Extra { name, .. } => {
-                    // Detect self-dependencies.
                     if dev.is_none() {
                         debug_assert!(
-                            source_name.is_none_or(|source_name| source_name != name),
+                            parent_name.is_none_or(|parent_name| parent_name != name),
                             "extras not flattened for {name}"
                         );
                     }
-                    PubGrubDependency {
+                    Self {
                         package,
                         version,
+                        parent: None,
                         url,
                     }
                 }
                 PubGrubPackageInner::Dev { name, .. } => {
-                    // Detect self-dependencies.
                     if dev.is_none() {
                         debug_assert!(
-                            source_name.is_none_or(|source_name| source_name != name),
+                            parent_name.is_none_or(|parent_name| parent_name != name),
                             "group not flattened for {name}"
                         );
                     }
-                    PubGrubDependency {
+                    Self {
                         package,
                         version,
+                        parent: None,
                         url,
                     }
                 }
@@ -135,6 +163,14 @@ impl PubGrubDependency {
             }
         })
     }
+
+    /// Extracts a possible conflicting item from this dependency.
+    ///
+    /// If this package can't possibly be classified as conflicting, then this
+    /// returns `None`.
+    pub(crate) fn conflicting_item(&self) -> Option<ConflictItemRef<'_>> {
+        self.package.conflicting_item()
+    }
 }
 
 /// A PubGrub-compatible package and version range.
@@ -227,7 +263,7 @@ impl PubGrubRequirement {
         extra: Option<ExtraName>,
         group: Option<GroupName>,
         requirement: &Requirement,
-    ) -> PubGrubRequirement {
+    ) -> Self {
         Self {
             package: PubGrubPackage::from_package(
                 requirement.name.clone(),
