@@ -1,13 +1,13 @@
 use std::fmt::{Display, Formatter};
 
+use crate::resolver::{MetadataUnavailable, VersionFork};
 use uv_distribution_types::IncompatibleDist;
 use uv_pep440::{Version, VersionSpecifiers};
-
-use crate::resolver::{MetadataUnavailable, VersionFork};
+use uv_platform_tags::{AbiTag, Tags};
 
 /// The reason why a package or a version cannot be used.
 #[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) enum UnavailableReason {
+pub enum UnavailableReason {
     /// The entire package cannot be used.
     Package(UnavailablePackage),
     /// A single version cannot be used.
@@ -29,7 +29,7 @@ impl Display for UnavailableReason {
 /// Most variant are from [`MetadataResponse`] without the error source, since we don't format
 /// the source and we want to merge unavailable messages across versions.
 #[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) enum UnavailableVersion {
+pub enum UnavailableVersion {
     /// Version is incompatible because it has no usable distributions
     IncompatibleDist(IncompatibleDist),
     /// The wheel metadata was found, but could not be parsed.
@@ -48,12 +48,12 @@ pub(crate) enum UnavailableVersion {
 impl UnavailableVersion {
     pub(crate) fn message(&self) -> String {
         match self {
-            UnavailableVersion::IncompatibleDist(invalid_dist) => format!("{invalid_dist}"),
-            UnavailableVersion::InvalidMetadata => "invalid metadata".into(),
-            UnavailableVersion::InconsistentMetadata => "inconsistent metadata".into(),
-            UnavailableVersion::InvalidStructure => "an invalid package format".into(),
-            UnavailableVersion::Offline => "to be downloaded from a registry".into(),
-            UnavailableVersion::RequiresPython(requires_python) => {
+            Self::IncompatibleDist(invalid_dist) => format!("{invalid_dist}"),
+            Self::InvalidMetadata => "invalid metadata".into(),
+            Self::InconsistentMetadata => "inconsistent metadata".into(),
+            Self::InvalidStructure => "an invalid package format".into(),
+            Self::Offline => "to be downloaded from a registry".into(),
+            Self::RequiresPython(requires_python) => {
                 format!("Python {requires_python}")
             }
         }
@@ -61,23 +61,40 @@ impl UnavailableVersion {
 
     pub(crate) fn singular_message(&self) -> String {
         match self {
-            UnavailableVersion::IncompatibleDist(invalid_dist) => invalid_dist.singular_message(),
-            UnavailableVersion::InvalidMetadata => format!("has {self}"),
-            UnavailableVersion::InconsistentMetadata => format!("has {self}"),
-            UnavailableVersion::InvalidStructure => format!("has {self}"),
-            UnavailableVersion::Offline => format!("needs {self}"),
-            UnavailableVersion::RequiresPython(..) => format!("requires {self}"),
+            Self::IncompatibleDist(invalid_dist) => invalid_dist.singular_message(),
+            Self::InvalidMetadata => format!("has {self}"),
+            Self::InconsistentMetadata => format!("has {self}"),
+            Self::InvalidStructure => format!("has {self}"),
+            Self::Offline => format!("needs {self}"),
+            Self::RequiresPython(..) => format!("requires {self}"),
         }
     }
 
     pub(crate) fn plural_message(&self) -> String {
         match self {
-            UnavailableVersion::IncompatibleDist(invalid_dist) => invalid_dist.plural_message(),
-            UnavailableVersion::InvalidMetadata => format!("have {self}"),
-            UnavailableVersion::InconsistentMetadata => format!("have {self}"),
-            UnavailableVersion::InvalidStructure => format!("have {self}"),
-            UnavailableVersion::Offline => format!("need {self}"),
-            UnavailableVersion::RequiresPython(..) => format!("require {self}"),
+            Self::IncompatibleDist(invalid_dist) => invalid_dist.plural_message(),
+            Self::InvalidMetadata => format!("have {self}"),
+            Self::InconsistentMetadata => format!("have {self}"),
+            Self::InvalidStructure => format!("have {self}"),
+            Self::Offline => format!("need {self}"),
+            Self::RequiresPython(..) => format!("require {self}"),
+        }
+    }
+
+    pub(crate) fn context_message(
+        &self,
+        tags: Option<&Tags>,
+        requires_python: Option<AbiTag>,
+    ) -> Option<String> {
+        match self {
+            Self::IncompatibleDist(invalid_dist) => {
+                invalid_dist.context_message(tags, requires_python)
+            }
+            Self::InvalidMetadata => None,
+            Self::InconsistentMetadata => None,
+            Self::InvalidStructure => None,
+            Self::Offline => None,
+            Self::RequiresPython(..) => None,
         }
     }
 }
@@ -91,14 +108,12 @@ impl Display for UnavailableVersion {
 impl From<&MetadataUnavailable> for UnavailableVersion {
     fn from(reason: &MetadataUnavailable) -> Self {
         match reason {
-            MetadataUnavailable::Offline => UnavailableVersion::Offline,
-            MetadataUnavailable::InvalidMetadata(_) => UnavailableVersion::InvalidMetadata,
-            MetadataUnavailable::InconsistentMetadata(_) => {
-                UnavailableVersion::InconsistentMetadata
-            }
-            MetadataUnavailable::InvalidStructure(_) => UnavailableVersion::InvalidStructure,
+            MetadataUnavailable::Offline => Self::Offline,
+            MetadataUnavailable::InvalidMetadata(_) => Self::InvalidMetadata,
+            MetadataUnavailable::InconsistentMetadata(_) => Self::InconsistentMetadata,
+            MetadataUnavailable::InvalidStructure(_) => Self::InvalidStructure,
             MetadataUnavailable::RequiresPython(requires_python, _python_version) => {
-                UnavailableVersion::RequiresPython(requires_python.clone())
+                Self::RequiresPython(requires_python.clone())
             }
         }
     }
@@ -106,7 +121,7 @@ impl From<&MetadataUnavailable> for UnavailableVersion {
 
 /// The package is unavailable and cannot be used.
 #[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) enum UnavailablePackage {
+pub enum UnavailablePackage {
     /// Index lookups were disabled (i.e., `--no-index`) and the package was not found in a flat index (i.e. from `--find-links`).
     NoIndex,
     /// Network requests were disabled (i.e., `--offline`), and the package was not found in the cache.
@@ -122,21 +137,21 @@ pub(crate) enum UnavailablePackage {
 impl UnavailablePackage {
     pub(crate) fn message(&self) -> &'static str {
         match self {
-            UnavailablePackage::NoIndex => "not found in the provided package locations",
-            UnavailablePackage::Offline => "not found in the cache",
-            UnavailablePackage::NotFound => "not found in the package registry",
-            UnavailablePackage::InvalidMetadata(_) => "invalid metadata",
-            UnavailablePackage::InvalidStructure(_) => "an invalid package format",
+            Self::NoIndex => "not found in the provided package locations",
+            Self::Offline => "not found in the cache",
+            Self::NotFound => "not found in the package registry",
+            Self::InvalidMetadata(_) => "invalid metadata",
+            Self::InvalidStructure(_) => "an invalid package format",
         }
     }
 
     pub(crate) fn singular_message(&self) -> String {
         match self {
-            UnavailablePackage::NoIndex => format!("was {self}"),
-            UnavailablePackage::Offline => format!("was {self}"),
-            UnavailablePackage::NotFound => format!("was {self}"),
-            UnavailablePackage::InvalidMetadata(_) => format!("has {self}"),
-            UnavailablePackage::InvalidStructure(_) => format!("has {self}"),
+            Self::NoIndex => format!("was {self}"),
+            Self::Offline => format!("was {self}"),
+            Self::NotFound => format!("was {self}"),
+            Self::InvalidMetadata(_) => format!("has {self}"),
+            Self::InvalidStructure(_) => format!("has {self}"),
         }
     }
 }
