@@ -7,13 +7,12 @@ use anyhow::Result;
 use rustc_hash::FxHashSet;
 
 use uv_cache::Cache;
-use uv_configuration::{
-    BuildKind, BuildOptions, BuildOutput, ConfigSettings, PackageConfigSettings, SourceStrategy,
-};
+use uv_configuration::{BuildKind, BuildOptions, BuildOutput, SourceStrategy};
 use uv_distribution_filename::DistFilename;
 use uv_distribution_types::{
-    CachedDist, DependencyMetadata, DistributionId, IndexCapabilities, IndexLocations,
-    InstalledDist, IsBuildBackendError, Requirement, Resolution, SourceDist,
+    CachedDist, ConfigSettings, DependencyMetadata, DistributionId, ExtraBuildRequires,
+    ExtraBuildVariables, IndexCapabilities, IndexLocations, InstalledDist, IsBuildBackendError,
+    PackageConfigSettings, Requirement, Resolution, SourceDist,
 };
 use uv_git::GitResolver;
 use uv_pep508::PackageName;
@@ -62,8 +61,10 @@ use crate::BuildArena;
 pub trait BuildContext {
     type SourceDistBuilder: SourceBuildTrait;
 
+    // Note: this function is async deliberately, because downstream code may need to
+    // run async code to get the interpreter, to resolve the Python version.
     /// Return a reference to the interpreter.
-    fn interpreter(&self) -> &Interpreter;
+    fn interpreter(&self) -> impl Future<Output = &Interpreter> + '_;
 
     /// Return a reference to the cache.
     fn cache(&self) -> &Cache;
@@ -101,6 +102,12 @@ pub trait BuildContext {
     /// Workspace discovery caching.
     fn workspace_cache(&self) -> &WorkspaceCache;
 
+    /// Get the extra build requirements.
+    fn extra_build_requires(&self) -> &ExtraBuildRequires;
+
+    /// Get the extra build variables.
+    fn extra_build_variables(&self) -> &ExtraBuildVariables;
+
     /// Resolve the given requirements into a ready-to-install set of package versions.
     fn resolve<'a>(
         &'a self,
