@@ -292,8 +292,7 @@ fn help_subcommand() {
     Commands:
       list          List the available Python installations
       install       Download and install Python versions
-      upgrade       Upgrade installed Python versions to the latest supported patch release (requires
-                    the `--preview` flag)
+      upgrade       Upgrade installed Python versions
       find          Search for a Python installation
       pin           Pin to a specific Python version
       dir           Show the uv Python installation directory
@@ -731,8 +730,7 @@ fn help_flag_subcommand() {
     Commands:
       list          List the available Python installations
       install       Download and install Python versions
-      upgrade       Upgrade installed Python versions to the latest supported patch release (requires
-                    the `--preview` flag)
+      upgrade       Upgrade installed Python versions
       find          Search for a Python installation
       pin           Pin to a specific Python version
       dir           Show the uv Python installation directory
