@@ -39,7 +39,7 @@ mod lock_conflict;
 
 mod lock_scenarios;
 
-mod version;
+mod network;
 
 #[cfg(all(feature = "python", feature = "pypi"))]
 mod pip_check;
@@ -78,12 +78,18 @@ mod python_find;
 #[cfg(feature = "python")]
 mod python_list;
 
+#[cfg(all(feature = "python", feature = "pypi"))]
+mod python_module;
+
 #[cfg(feature = "python-managed")]
 mod python_install;
 
 #[cfg(feature = "python")]
 mod python_pin;
 
+#[cfg(feature = "python-managed")]
+mod python_upgrade;
+
 #[cfg(all(feature = "python", feature = "pypi"))]
 mod run;
 
@@ -120,7 +126,10 @@ mod tree;
 #[cfg(feature = "python")]
 mod venv;
 
+mod version;
+
 #[cfg(all(feature = "python", feature = "pypi"))]
 mod workflow;
 
+mod extract;
 mod workspace;
